import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateFaqPageSectionTwoDto} from "./dto/create-faq-page-section-two.dto";
import {FaqPageSectionTwoEntity} from "./entities/faq-page-section-two.entity";
import {UpdateFaqPageSectionTwoDto} from "./dto/update-faq-page-section-two.dto";

@EntityRepository(FaqPageSectionTwoEntity)
export class FaqPageSectionTwoRepository extends Repository<FaqPageSectionTwoEntity> {
    async createEntity(dto: CreateFaqPageSectionTwoDto): Promise<FaqPageSectionTwoEntity | Error> {
        try {
            const {
                image,
                title,
                description
            } = dto;
            const entity = this.create({
                image,
                title,
                description
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<FaqPageSectionTwoEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<FaqPageSectionTwoEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(FaqPageSectionTwoEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateFaqPageSectionTwoDto): Promise<FaqPageSectionTwoEntity | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                title: dto.title,
                description: dto.description
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}