import { Module } from "@nestjs/common";
import { JwtModule } from "@nestjs/jwt";
import { TypeOrmModule } from "@nestjs/typeorm";
import JwtConfigService from "src/core/jwt/jwt-config.service";
import JwtHelper from "src/core/jwt/jwt.helper";
import { AuthService } from "../auth/auth.service";
import { CareerRepository } from "../career/career.repository";
import { ChangePasswordTokenRepository } from "../change-password-token/change-password-token.repository";
import { UsersRepository } from "../users/user.repository";
import { EmailNotifierRepository } from "../email-notifier/email-notifier-repository";
import { SiteFrontendController } from "./site-frontend.controller";
import { SiteFrontendService } from "./site-frontend.service";
import { MailSenderService } from "../mail-sender/mail-sender.service";
import { CareerDepartmentRepository } from "../career-department/career-department.repository";
import { CareerListPageCmsRepository } from "../CMS/career-list-page-cms/career-list-page-cms.repository";
import { CareerDetailsPageCmsRepository } from "../CMS/career-details-page-cms/career-details-page-cms.repository";
import { WhyJoinUsRepository } from "../why-join-us/why-join-us.repository";
import { ScheduleVendorDayBannerCmsRepository } from "../CMS/schedule-vendor-day-banner-cms/schedule-vendor-day-banner-cms.repository";
import { BlogCategoryRepository } from "../blog/blog-category/blog-category.repository";
import { BlogTagRepository } from "../blog/blog-tag/blog-tag.repository";
import { BlogPostRepository } from "../blog/blog-post/blog-post.repository";
import { BlogPostTagsRepository } from "../blog/blog-post-tags/blog-post-tags.repository";
import { FAQRepository } from "../faq/faq.repository";
import { FaqPageSectionTwoRepository } from "../faq-page-section-two/faq-page-section-two.repository";
import { FAQPageCmsRepository } from "../CMS/faq-page-cms/faq-page-cms.repository";
import { ContactUsPageCmsRepository } from "../CMS/contact-us-page-cms/contact-us-page-cms.repository";
import { ContactUsSectionOneRepository } from "../contact-us-section-one/contact-us-section-one.repository";

@Module({
    imports: [
        JwtModule.registerAsync({
            useClass: JwtConfigService
        }),
        TypeOrmModule.forFeature([UsersRepository]),
        TypeOrmModule.forFeature([ChangePasswordTokenRepository]),
        TypeOrmModule.forFeature([
            EmailNotifierRepository,
            CareerRepository,
            CareerDepartmentRepository,
            CareerListPageCmsRepository,
            CareerDetailsPageCmsRepository,
            WhyJoinUsRepository,
            ScheduleVendorDayBannerCmsRepository,
            BlogCategoryRepository,
            BlogTagRepository,
            BlogPostRepository,
            BlogPostTagsRepository,
            FAQRepository,
            FaqPageSectionTwoRepository,
            FAQPageCmsRepository,
            ContactUsPageCmsRepository,
            ContactUsSectionOneRepository
        ])
    ],
    controllers: [SiteFrontendController],
    providers: [
        SiteFrontendService,
        AuthService,
        JwtHelper,
        MailSenderService
    ]
})
export class SiteFrontendModule {}
