import { Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { ResponseUtils } from "src/utils/response.utils";
import { CareerRepository } from "../career/career.repository";
import { CareerDepartmentRepository } from "../career-department/career-department.repository";
import { FindOperator, Like } from "typeorm";
import { CareerListPageCmsRepository } from "../CMS/career-list-page-cms/career-list-page-cms.repository";
import { CareerDetailsPageCmsRepository } from "../CMS/career-details-page-cms/career-details-page-cms.repository";
import { WhyJoinUsRepository } from "../why-join-us/why-join-us.repository";
import { ScheduleVendorDayBannerCmsRepository } from "../CMS/schedule-vendor-day-banner-cms/schedule-vendor-day-banner-cms.repository";
import { BlogCategoryRepository } from "../blog/blog-category/blog-category.repository";
import { BlogTagRepository } from "../blog/blog-tag/blog-tag.repository";
import { BlogPostRepository } from "../blog/blog-post/blog-post.repository";
import { UpdateBlogPostDto } from "../blog/blog-post/dto/update-blog-post.dto";
import { BlogPostTagsRepository } from "../blog/blog-post-tags/blog-post-tags.repository";
import { FAQRepository } from "../faq/faq.repository";
import { FaqPageSectionTwoRepository } from "../faq-page-section-two/faq-page-section-two.repository";
import { FAQPageCmsRepository } from "../CMS/faq-page-cms/faq-page-cms.repository";
import { ContactUsPageCmsRepository } from "../CMS/contact-us-page-cms/contact-us-page-cms.repository";
import { ContactUsSectionOneRepository } from "../contact-us-section-one/contact-us-section-one.repository";

@Injectable()
export class SiteFrontendService {
    constructor(
        @InjectRepository(CareerRepository)
        private readonly careerRepository: CareerRepository,
        @InjectRepository(CareerDepartmentRepository)
        private readonly careerDepartmentRepository: CareerDepartmentRepository,
        @InjectRepository(CareerListPageCmsRepository)
        private readonly careerListPageCmsRepository: CareerListPageCmsRepository,
        @InjectRepository(CareerDetailsPageCmsRepository)
        private readonly careerDetailsPageCmsRepository: CareerDetailsPageCmsRepository,
        @InjectRepository(WhyJoinUsRepository)
        private readonly whyJoinUsRepository: WhyJoinUsRepository,
        @InjectRepository(ScheduleVendorDayBannerCmsRepository)
        private readonly scheduleVendorDayBannerCmsRepository: ScheduleVendorDayBannerCmsRepository,
        @InjectRepository(BlogCategoryRepository)
        private readonly blogCategoryRepository: BlogCategoryRepository,
        @InjectRepository(BlogTagRepository)
        private readonly blogTagRepository: BlogTagRepository,
        @InjectRepository(BlogPostRepository)
        private readonly blogPostRepository: BlogPostRepository,
        @InjectRepository(BlogPostTagsRepository)
        private readonly blogPostTagsRepository: BlogPostTagsRepository,
        @InjectRepository(FAQRepository)
        private readonly faqRepository: FAQRepository,
        @InjectRepository(FaqPageSectionTwoRepository)
        private readonly faqSecTwoRepository: FaqPageSectionTwoRepository,
        @InjectRepository(FAQPageCmsRepository)
        private readonly faqCmsRepository: FAQPageCmsRepository,
        @InjectRepository(ContactUsPageCmsRepository)
        private readonly contactUsPageCmsRepository: ContactUsPageCmsRepository,
        @InjectRepository(ContactUsSectionOneRepository)
        private readonly contactUsSectionOneRepository: ContactUsSectionOneRepository
    ) {}

    async getCareerPageData(): Promise<any | Error> {
        const careerDepartmentList = await this.careerDepartmentRepository.findByFilterQuery({});
        const totalCount = await this.careerRepository.totalCareerCount();
        let careerDepartmentData: any = [];
        for (let i = 0; i < Object.keys(careerDepartmentList).length; i++) {
            const careerCount = await this.careerRepository.countByDepartmentName(careerDepartmentList[i].name);
            careerDepartmentList[i]["count"] = careerCount;
            careerDepartmentData.push(careerDepartmentList[i]);
        }

        const response = {
            careerDepartmentData: careerDepartmentData,
            totalCount: totalCount
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getCareerDetailsData(id: string): Promise<any | Error> {
        const cmsData = await this.careerDetailsPageCmsRepository.findAll();
        const careerDetailsData = await this.careerRepository.findByFilterQuery({ id: id });
        const whyJoinUsData = await this.whyJoinUsRepository.findByFilterQuery({});
        const latestCareerList = await this.careerRepository.getLatestCareerList();

        const response = {
            cmsData: cmsData,
            careerDetailsData: careerDetailsData,
            whyJoinUsData: whyJoinUsData,
            latestCareerList: latestCareerList
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }

    async getCareerDepartmentWiseData(
        departmentNameSlug: any,
        page: number,
        limit: number
    ): Promise<any | Error> {
        type FindOption = {
            departmentName?: FindOperator<string>;
        };

        const findOptions: FindOption = {};

        let departmentData = await this.careerDepartmentRepository.findBySlug(departmentNameSlug);
        let departmentName: any;
        if (Object.keys(departmentData).length > 0) {
            departmentName = departmentData[0].name;
        }

        if (departmentName) findOptions.departmentName = departmentName;

        const cmsData = await this.careerListPageCmsRepository.findAll();
        const whyJoinUsData = await this.whyJoinUsRepository.findByFilterQuery({});
        const careerList = await this.careerRepository.findAllWithPagination(findOptions, page, limit);
        const isCareerDepartmentExist = await this.careerDepartmentRepository.findOne({
            where: {
                name: departmentName,
                isDeleted: false
            }
        });

        let data = {
            cmsData: cmsData,
            careerList: careerList,
            whyJoinUsData: whyJoinUsData,
            isCareerDepartmentExist: typeof isCareerDepartmentExist !== "undefined" ? true : false
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async getBlogData(
        page: number,
        limit: number,
        keyword: any,
        tagName: any,
        categoryId: any
    ): Promise<any | Error> {
        type FindOption = {
            title?: FindOperator<string>;
            categoryId?: FindOperator<string>;
        };

        const findOptions: FindOption = {};

        if (keyword) findOptions.title = Like("%" + keyword + "%");
        if (categoryId) findOptions.categoryId = categoryId;

        if (tagName) {
            const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
            const blogCategoryList = await this.blogCategoryRepository.findByFilterQuery({});
            const blogTagList = await this.blogTagRepository.findByFilterQuery({});
            const topBlogList = await this.blogPostRepository.getTopBlogList();
            const totalBlogCount = await this.blogPostRepository.countAllBlogs();
            const blogList = await this.blogPostRepository.findBlogByTagName(
                page,
                limit,
                tagName
            );

            let data = {
                scheduleVendorDayData: scheduleVendorDayData,
                blogCategoryList: blogCategoryList,
                blogTagList: blogTagList,
                topBlogList: topBlogList,
                blogList: blogList,
                totalBlogCount: totalBlogCount
            };

            return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
        } else {
            const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
            const blogCategoryList = await this.blogCategoryRepository.findByFilterQuery({});
            const blogTagList = await this.blogTagRepository.findByFilterQuery({});
            const topBlogList = await this.blogPostRepository.getTopBlogList();
            const totalBlogCount = await this.blogPostRepository.countAllBlogs();
            const blogList = await this.blogPostRepository.findBlogByFilterQuery(
                page,
                limit,
                findOptions
            );

            let data = {
                scheduleVendorDayData: scheduleVendorDayData,
                blogCategoryList: blogCategoryList,
                blogTagList: blogTagList,
                topBlogList: topBlogList,
                blogList: blogList,
                totalBlogCount: totalBlogCount
            };

            return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
        }
    }

    async getBlogDetailsData(urlSlug: string): Promise<any | Error> {
        const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
        const blogData = await this.blogPostRepository.findOneEntityBySlug(urlSlug);
        let id: any;
        if (Object.keys(blogData).length > 0) {
            id = blogData[0].id;
            let viewCount = 0;
            let totalViews = blogData[0].totalViews;
            let postId = blogData[0].id;
            viewCount = totalViews + 1;
            let blogPostDto = {
                totalViews: viewCount
            };
            await this.blogPostRepository.updateBlogViewCount(
                postId,
                blogPostDto as UpdateBlogPostDto
            );
        }
        const blogTags = await this.blogPostTagsRepository.findBlogPostTagsByPostId(id);
        if (blogTags) {
            blogData[0]["tags"] = blogTags;
        }

        let data = {
            scheduleVendorDayData: scheduleVendorDayData,
            blogData: blogData
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async getFaqPageData(): Promise<any | Error> {
        const faqCms = await this.faqCmsRepository.findAll();
        const faqSectionTwo = await this.faqSecTwoRepository.findByFilterQuery({});
        const faq = await this.faqRepository.findByFilterQuery({});

        const response = { faqCms, faqSectionTwo, faq };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getContactUsPageData(): Promise<any | Error> {
        const cmsData = await this.contactUsPageCmsRepository.findAll();
        const sectionOneData = await this.contactUsSectionOneRepository.findByFilterQuery({});

        const response = { 
            cmsData: cmsData, 
            sectionOneData: sectionOneData
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }
}
