import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateWhyJoinUsDto } from "./dto/create-why-join-us.dto";
import { WhyJoinUsEntity } from "./entities/why-join-us.entity";
import {UpdateWhyJoinUsDto} from "./dto/update-why-join-us.dto";

@EntityRepository(WhyJoinUsEntity)
export class WhyJoinUsRepository extends Repository<WhyJoinUsEntity> {
    async createEntity(dto: CreateWhyJoinUsDto): Promise<WhyJoinUsEntity | Error> {
        try {
            const {
                icon,
                title,
                description
            } = dto;
            const entity = this.create({
                icon,
                title,
                description
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<WhyJoinUsEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<WhyJoinUsEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(WhyJoinUsEntity)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateWhyJoinUsDto): Promise<WhyJoinUsEntity | Error> {
        try {
            await this.update(id, {
                icon: dto.icon,
                title: dto.title,
                description: dto.description
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}